'use strict';

var _reduxMockStore = require('redux-mock-store');

var _reduxMockStore2 = _interopRequireDefault(_reduxMockStore);

var _normalizr = require('normalizr');

var _ = require('./');

var _2 = _interopRequireDefault(_);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

describe('normalize middleware', function () {
  var mockStore = (0, _reduxMockStore2.default)([_2.default]);
  var store = mockStore();

  afterEach(function () {
    store.clearActions();
  });

  var response = {
    data: [{
      id: 1,
      title: 'Article 1',
      author: {
        id: 10,
        name: 'John'
      }
    }, {
      id: 2,
      title: 'Article 2',
      author: {
        id: 20,
        name: 'Alex'
      }
    }]
  };

  var responseWithNestedData = {
    data: { items: response.data }
  };

  var authorSchema = new _normalizr.schema.Entity('authors');
  var articlesSchema = [new _normalizr.schema.Entity('articles', {
    author: authorSchema
  })];

  var action = {
    type: 'success',
    payload: response,
    meta: {
      previousAction: {
        normalize: {
          schema: articlesSchema
        }
      }
    }
  };

  var actionWithPath = {
    type: 'success',
    payload: responseWithNestedData,
    meta: {
      previousAction: {
        normalize: {
          schema: articlesSchema,
          path: 'items'
        }
      }
    }
  };

  var normalizedArticles = {
    1: {
      id: 1,
      title: 'Article 1',
      author: 10
    },
    2: {
      id: 2,
      title: 'Article 2',
      author: 20
    }
  };

  var normalizedAuthors = {
    10: {
      id: 10,
      name: 'John'
    },
    20: {
      id: 20,
      name: 'Alex'
    }
  };

  it('should normalize items according to schema', function () {
    store.dispatch(action);
    expect(store.getActions()[0].payload.data.articles.items).toEqual(normalizedArticles);
    expect(store.getActions()[0].payload.data.authors.items).toEqual(normalizedAuthors);
  });

  it('should include ids of items', function () {
    store.dispatch(actionWithPath);
    expect(store.getActions()[0].payload.data.articles.allIds).toEqual([1, 2]);
    expect(store.getActions()[0].payload.data.authors.allIds).toEqual([10, 20]);
  });

  it('should normalize data from path', function () {
    store.dispatch(action);
    expect(store.getActions()[0].payload.data.articles.items).toEqual(normalizedArticles);
    expect(store.getActions()[0].payload.data.authors.items).toEqual(normalizedAuthors);
  });

  it('should not try to normalize data if no schema provided', function () {
    var actionWithoutSchema = {
      type: 'success',
      payload: response
    };

    store.dispatch(actionWithoutSchema);
    expect(store.getActions()[0]).toEqual(actionWithoutSchema);
  });

  it('should not try to normalize data if no payload provided', function () {
    var actionWithoutSchema = {
      type: 'success',
      meta: {
        previousAction: {
          schema: articlesSchema
        }
      }
    };

    store.dispatch(actionWithoutSchema);
    expect(store.getActions()[0]).toEqual(actionWithoutSchema);
  });
});